"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculatePrices = calculatePrices;

var _claims = require("./claims");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Price = function Price() {
  _classCallCheck(this, Price);

  this.subtotal = 0;
  this.discount = 0;
  this.couponDiscount = 0;
  this.memberDiscount = 0;
  this.discountItems = [];
  this.netTotal = 0;
  this.taxes = 0;
  this.total = 0;
};

function findTax(settings, country, type) {
  if (settings && settings.taxes) {
    for (var i in settings.taxes) {
      var tax = settings.taxes[i];
      if ((tax.countries == null || tax.countries.indexOf(country) > -1) && (tax.product_types == null || tax.product_types.indexOf(type) > -1)) {
        return tax;
      }
    }
  }
  return null;
}

function couponValidFor(claims, coupon, item) {
  if (!(0, _claims.checkClaims)(claims, coupon.claims)) {
    return false;
  }
  if (coupon.product_types && coupon.product_types.length) {
    return coupon.product_types.indexOf(item.type) > -1;
  }
  if (coupon.products && coupon.products.length) {
    return coupon.products.indexOf(item.sku) > -1;
  }
  return true;
}

function fixedAmount(amounts, currency) {
  var fixed = amounts && amounts.filter(function (amount) {
    return amount.currency === currency;
  })[0];
  return fixed && Math.round(parseFloat(fixed.amount) * 100) || 0;
}

function discountItem(type) {
  var percentage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var fixed = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

  return {
    type: type,
    percentage: percentage,
    fixed: fixed
  };
}

function calculateDiscount(amountToDiscount) {
  var percentage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var fixed = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

  var discount = 0;
  if (percentage > 0) {
    discount = Math.round(amountToDiscount * percentage / 100);
  }
  discount += fixed;

  if (discount > amountToDiscount) {
    return amountToDiscount;
  }
  return discount;
}

function calculateTaxes(amountToTax, originalPrice, item, settings, country) {
  var includeTaxes = settings && settings.prices_include_taxes;
  var ratio = amountToTax / originalPrice;

  var taxAmounts = [];
  if (item.vat) {
    taxAmounts.push({ price: amountToTax, percentage: parseInt(item.vat, 10) });
  } else if (settings && item.price.items && item.price.items.length) {
    item.price.items.forEach(function (priceItem) {
      var realPrice = priceItem.cents * ratio;
      var tax = findTax(settings, country, priceItem.type);
      taxAmounts.push({
        price: Math.round(realPrice),
        percentage: tax ? tax.percentage : 0
      });
    });
  } else {
    var tax = findTax(settings, country, item.type);
    if (tax) {
      taxAmounts.push({ price: amountToTax, percentage: tax.percentage });
    }
  }

  var taxes = 0;
  var netTotal = 0;
  if (taxAmounts.length) {
    taxAmounts.forEach(function (tax) {
      if (includeTaxes) {
        var amount = Math.round(tax.price / (100 + tax.percentage) * 100 * (tax.percentage / 100));
        tax.price -= amount;
        taxes += amount;
      } else {
        taxes += Math.round(tax.price * tax.percentage / 100);
      }
      netTotal += tax.price;
    });
  } else {
    netTotal = amountToTax;
  }

  return {
    taxes: taxes,
    netTotal: netTotal
  };
}

function calculatePrices(settings, claims, country, currency, coupon, items) {
  var price = new Price();
  price.items = [];
  items && items.forEach(function (item) {
    var itemPrice = new Price();
    itemPrice.quantity = item.quantity || 1;

    var originalPrice = item.price.cents + (item.addonPrice ? item.addonPrice.cents : 0);

    var _calculateTaxes = calculateTaxes(originalPrice, originalPrice, item, settings, country),
        subtotal = _calculateTaxes.netTotal;

    itemPrice.subtotal = subtotal;

    if (coupon && couponValidFor(claims, coupon, item)) {
      itemPrice.discount = calculateDiscount(originalPrice, coupon.percentage, fixedAmount(coupon.fixed, currency));
      itemPrice.couponDiscount = itemPrice.discount;
      itemPrice.discountItems.push(discountItem("coupon", coupon.percentage, fixedAmount(coupon.fixed, currency)));
    }
    if (settings && settings.member_discounts) {
      settings.member_discounts.forEach(function (discount) {
        if (couponValidFor(claims, discount, item)) {
          var memberDiscount = calculateDiscount(originalPrice, discount.percentage, fixedAmount(discount.fixed, currency));
          itemPrice.discount = itemPrice.discount || 0;
          itemPrice.discount += memberDiscount;
          itemPrice.memberDiscount = itemPrice.memberDiscount || 0;
          itemPrice.memberDiscount += memberDiscount;
          itemPrice.discountItems.push(discountItem("member", discount.percentage, fixedAmount(discount.fixed, currency)));
        }
      });
    }

    var discountedPrice = Math.max(0, originalPrice - itemPrice.discount);

    var _calculateTaxes2 = calculateTaxes(discountedPrice, originalPrice, item, settings, country),
        taxes = _calculateTaxes2.taxes,
        netTotal = _calculateTaxes2.netTotal;

    itemPrice.taxes = taxes;
    itemPrice.netTotal = netTotal;
    itemPrice.total = itemPrice.netTotal + itemPrice.taxes;
    price.items.push(itemPrice);

    price.subtotal += itemPrice.subtotal * itemPrice.quantity;
    price.discount += itemPrice.discount * itemPrice.quantity;
    price.couponDiscount += itemPrice.couponDiscount * itemPrice.quantity;
    price.memberDiscount += itemPrice.memberDiscount * itemPrice.quantity;
    price.netTotal += itemPrice.netTotal * itemPrice.quantity;
    price.taxes += itemPrice.taxes * itemPrice.quantity;
    price.total += itemPrice.total * itemPrice.quantity;
  });

  return price;
}